from bottle import route, redirect

from matemat.db import MatematDatabase
from matemat.webserver import template, session
from matemat.webserver.config import get_app_config, get_stock_provider


@route('/')
def main_page():
    """
    The main page, showing either the user list (if no user is logged in) or the product list (if a user is logged in).
    """
    config = get_app_config()
    session_id: str = session.start()
    with MatematDatabase(config['DatabaseFile']) as db:
        # Check whether a user is logged in
        if session.has(session_id, 'authenticated_user'):
            # Fetch the user id and authentication level (touchkey vs password) from the session storage
            uid: int = session.get(session_id, 'authenticated_user')
            authlevel: int = session.get(session_id, 'authentication_level')
            # Fetch the user object from the database (for name display, price calculation and admin check)
            user = db.get_user(uid)
            # Fetch the list of products to display
            products = db.list_products()
            # Prepare a response with a jinja2 template
            return template.render('productlist.html',
                                   authuser=user, products=products, authlevel=authlevel, stock=get_stock_provider(),
                                   setupname=config['InstanceName'])
        else:
            # If there are no admin users registered, jump to the admin creation procedure
            if not db.has_admin_users():
                redirect('/userbootstrap')
            # If no user is logged in, fetch the list of users and render the userlist template
            users = db.list_users(with_touchkey=True)
            return template.render('userlist.html',
                                   users=users, setupname=config['InstanceName'])
